package com.faraphel.tasks_valider.database.api.entities.base

import com.faraphel.tasks_valider.database.dao.base.BaseCronDao
import com.faraphel.tasks_valider.database.entities.base.BaseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD

/**
 * A base for the API to handle the database operations.
 * This is preconfigured to handle JSON data.
 * @param Entity the entity type to handle
 */
abstract class BaseJsonApi<Entity: BaseEntity>(private val dao: BaseCronDao<Entity>) : BaseApi {
    companion object {
        private val parser = Gson()  ///< The JSON parser
    }

    private val entityTypeToken: TypeToken<Entity> = object: TypeToken<Entity>() {}  ///< the type of the managed entity

    // Requests

    override fun head(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson<Entity>(
            httpSession.inputStream.bufferedReader().readText(),
            this.entityTypeToken.type
        )
        val exists = this.dao.exists(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (exists) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            if (exists) "Exists" else "Not found"
        )
    }

    override fun get(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.OK,
            "application/json",
            parser.toJson(this.dao.getAll())
        )
    }

    override fun post(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson<Entity>(
            httpSession.inputStream.bufferedReader().readText(),
            this.entityTypeToken.type
        )
        val id = this.dao.insert(obj)

        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.CREATED,
            "text/plain",
            id.toString()
        )
    }

    override fun delete(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson<Entity>(
            httpSession.inputStream.bufferedReader().readText(),
            this.entityTypeToken.type
        )
        val count = this.dao.delete(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (count > 0) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            count.toString()
        )
    }
}