package com.faraphel.tasks_valider.database.api.entities.base

import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.SessionEntity
import com.faraphel.tasks_valider.database.entities.base.BaseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD

/**
 * A base for the API to handle the database operations.
 * This is preconfigured to handle data for Task objects.
 * @param Entity the entity type to handle
 */
abstract class BaseTaskApi<Entity: BaseEntity>(
    private val dao: BaseTaskDao<Entity>,
    private val session: SessionEntity,
) : BaseJsonApi<Entity>(dao) {
    companion object {
        private val parser = Gson()  ///< The JSON parser
    }

    private val entityTypeToken: TypeToken<Entity> = object: TypeToken<Entity>() {}  ///< the type of the managed entity

    // Requests

    override fun head(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        val obj = parser.fromJson<Entity>(
            httpSession.inputStream.bufferedReader().readText(),
            this.entityTypeToken.type
        )
        // check if the object is in the object accessible from the session
        val exists = this.dao.getAllBySession(session.id).contains(obj)

        return NanoHTTPD.newFixedLengthResponse(
            if (exists) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            if (exists) "Exists" else "Not found"
        )
    }

    override fun get(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.OK,
            "application/json",
            parser.toJson(this.dao.getAllBySession(session.id))
        )
    }
}