package com.faraphel.tasks_valider.database.dao

import androidx.room.Dao
import androidx.room.Query
import com.faraphel.tasks_valider.database.dao.base.BaseCronDao
import com.faraphel.tasks_valider.database.dao.base.BaseSessionDao
import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.ClassEntity
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.faraphel.tasks_valider.database.entities.RelationClassPersonEntity
import com.faraphel.tasks_valider.database.entities.SessionEntity

@Dao
interface ClassDao : BaseTaskDao<ClassEntity> {
    @Query("SELECT * FROM ${ClassEntity.TABLE_NAME}")
    override fun getAll(): List<ClassEntity>

    @Query(
        "SELECT * FROM ${ClassEntity.TABLE_NAME} " +
        "JOIN ${SessionEntity.TABLE_NAME} " +
        "ON ${SessionEntity.TABLE_NAME}.class_id = ${ClassEntity.TABLE_NAME}.id " +
        "WHERE ${SessionEntity.TABLE_NAME}.id = :sessionId"
    )
    override fun getAllBySession(sessionId: Long): List<ClassEntity>

    /**
     * Get the object from its identifier
     */
    @Query("SELECT * FROM ${ClassEntity.TABLE_NAME} WHERE id = :id")
    fun getById(id: Long): ClassEntity?

    /**
     * Get all the sessions this class attended
     * @param id the id of the class
     */
    @Query("SELECT * FROM ${SessionEntity.TABLE_NAME} WHERE class_id = :id")
    fun getSessions(id: Long): List<SessionEntity>

    /**
     * Get all the students in a class
     * @param id the id of the class
     */
    @Query(
        "SELECT ${PersonEntity.TABLE_NAME}.* FROM ${PersonEntity.TABLE_NAME} " +
        "JOIN ${RelationClassPersonEntity.TABLE_NAME} " +
        "ON ${PersonEntity.TABLE_NAME}.id = ${RelationClassPersonEntity.TABLE_NAME}.student_id " +
        "WHERE ${RelationClassPersonEntity.TABLE_NAME}.class_id = :id"
    )
    fun getStudents(id: Long): List<PersonEntity>
}
