package com.faraphel.tasks_valider.database.dao

import androidx.room.Dao
import androidx.room.Query
import com.faraphel.tasks_valider.database.dao.base.BaseCronDao
import com.faraphel.tasks_valider.database.dao.base.BaseSessionDao
import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.*

@Dao
interface PersonDao : BaseTaskDao<PersonEntity> {
    @Query("SELECT * FROM ${PersonEntity.TABLE_NAME}")
    override fun getAll(): List<PersonEntity>

    /**
     * Get the object from its identifier
     */
    @Query("SELECT * FROM ${PersonEntity.TABLE_NAME} WHERE id = :id")
    fun getById(id: Long): PersonEntity?

    @Query(
        "SELECT * FROM ${PersonEntity.TABLE_NAME} " +
        "WHERE id IN (" +
            "SELECT student_id FROM ${RelationClassPersonEntity.TABLE_NAME} " +
            "JOIN ${ClassEntity.TABLE_NAME} " +
            "ON ${RelationClassPersonEntity.TABLE_NAME}.class_id = ${ClassEntity.TABLE_NAME}.id " +
            "JOIN ${SessionEntity.TABLE_NAME} " +
            "ON ${SessionEntity.TABLE_NAME}.class_id = ${ClassEntity.TABLE_NAME}.id " +
            "WHERE ${SessionEntity.TABLE_NAME}.id = :sessionId" +
        ")"
    )
    override fun getAllBySession(sessionId: Long): List<PersonEntity>

    /**
     * Allow to get all the classes the person is attending as a student
     */
    @Query(
        "SELECT * FROM ${ClassEntity.TABLE_NAME} " +
        "JOIN ${RelationClassPersonEntity.TABLE_NAME} " +
        "ON ${ClassEntity.TABLE_NAME}.id = ${RelationClassPersonEntity.TABLE_NAME}.student_id " +
        "WHERE ${RelationClassPersonEntity.TABLE_NAME}.student_id = :id"
    )
    fun getClasses(id: Long): List<ClassEntity>


    /**
     * Get all the tasks this user approved as a teacher
     * @param id the id of the person
     */
    @Query("SELECT * FROM ${ValidationEntity.TABLE_NAME} WHERE teacher_id = :id")
    fun getTasksApproved(id: Long): List<ValidationEntity>

    /**
     * Get all the tasks this user validated as a student
     * @param id the id of the person
     */
    @Query("SELECT * FROM ${ValidationEntity.TABLE_NAME} WHERE student_id = :id")
    fun getTasksValidated(id: Long): List<ValidationEntity>
}
