package com.faraphel.tasks_valider.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.PrimaryKey
import com.faraphel.tasks_valider.connectivity.task.session.TaskRole
import com.faraphel.tasks_valider.database.entities.base.BaseEntity
import kotlinx.serialization.Serializable
import java.security.MessageDigest
import java.util.*

@Serializable
@Entity(tableName = PersonEntity.TABLE_NAME)
data class PersonEntity (
    @ColumnInfo("id") @PrimaryKey(autoGenerate = true) val id: Long = 0,
    @ColumnInfo("first_name") val firstName: String,
    @ColumnInfo("last_name") val lastName: String,
    @ColumnInfo("card_id") val cardId: String? = null,
    @ColumnInfo("password_hash") val passwordHash: String? = null,
    @ColumnInfo("role") val role: TaskRole = TaskRole.STUDENT,
) : BaseEntity() {
    companion object {
        const val TABLE_NAME = "persons"

        /**
         * Hash a password
         */
        fun hashPassword(password: String): String {
            val digester = MessageDigest.getInstance("SHA-256")
            val hash = digester.digest(password.toByteArray())
            return hash.joinToString("") { byte -> "%02x".format(byte) }
        }
    }

    /**
     * Constructor with string password
     */
    constructor(
        firstName: String,
        lastName: String,
        cardId: String? = null,
        password: String? = null,
        role: TaskRole,
    ) : this(
        firstName = firstName,
        lastName = lastName,
        cardId = cardId,
        passwordHash = password?.let { hashPassword(password) },
        role = role,
    )

    /**
     * Return the full name of the person
     */
    fun fullName(): String = "${this.firstName.capitalize(Locale.ROOT)} ${this.lastName.uppercase()}"

    /**
     * Check if the password is correct
     */
    fun checkPassword(password: String): Boolean = this.passwordHash == hashPassword(password)
}
