package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.widget.Toast
import androidx.compose.foundation.layout.Column
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.MutableState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.database.entities.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken

/**
 * This screen represent a student
 * @param student the student object
 */
@Composable
fun TaskStudentScreen(activity: Activity, client: TaskClient, student: PersonEntity) {
    val tasks = remember { mutableStateOf<List<TaskEntity>?>(null) }

    if (tasks.value == null)
        Thread { refreshTasks(activity, client, student, tasks) }.start()

    Column {
        Text(text = student.fullName())

        tasks.value?.forEach { task ->
            Button(onClick = {}) {
                Column {
                    Text(task.title)
                    task.description?.let { description -> Text(description) }
                }
            }
        }
    }
}

fun refreshTasks(
    activity: Activity,
    client: TaskClient,
    student: PersonEntity,
    tasks: MutableState<List<TaskEntity>?>
) {
    val jsonParser = Gson()

    // try to obtain the list of subject
    val responseSubjects = client.get("entities/" + RelationPersonSessionSubjectEntity.TABLE_NAME)

    // in case of error, notify it
    if (!responseSubjects.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, responseSubjects.message, Toast.LENGTH_LONG).show() }

    // parse the list of subjects
    val allPersonSessionSubjects = jsonParser.fromJson<List<RelationPersonSessionSubjectEntity>>(
        responseSubjects.body.string(),
        object : TypeToken<List<RelationPersonSessionSubjectEntity>>(){}.type
    )

    // get the subject that the student is using
    val relationPersonSessionSubjects = allPersonSessionSubjects.firstOrNull { relation -> relation.studentId == student.id }

    if (relationPersonSessionSubjects == null)
        // TODO(Faraphel): should be able to assign a subject ?
        return activity.runOnUiThread { Toast.makeText(activity, "No subject assigned", Toast.LENGTH_LONG).show() }

    // try to obtain the list of tasks
    val responseTasks = client.get("entities/" + TaskEntity.TABLE_NAME)

    // in case of error, notify it
    if (!responseTasks.isSuccessful)
        return activity.runOnUiThread { Toast.makeText(activity, responseTasks.message, Toast.LENGTH_LONG).show() }

    // parse the list of subjects
    val allTasks = jsonParser.fromJson<List<TaskEntity>>(
        responseTasks.body.string(),
        object : TypeToken<List<TaskEntity>>(){}.type
    )

    // get the tasks that are linked to this subject
    tasks.value = allTasks.filter { task ->
        task.subjectId == relationPersonSessionSubjects.subjectId
    }.sortedBy { task ->
        task.order
    }
}
